MODULE = 'SQLExecutorX'

LIMIT_1 = 1

LIMIT_2 = 2

CACHE_SIZE = 64

MYSQL_PORT, POSTGRESQL_PORT = 3306, 5432

PARAM_DRIVER, PARAM_DEBUG, PARAM_SHOW_SQL, PARAM_POOL_SIZE, PARAM_PORT = 'driver', 'debug', 'show_sql', 'pool_size', 'port'
